<?php

/**
 * Fired during plugin deactivation
 *
 * @since      1.0.0
 *
 * @package    GPLVault_Updater
 * @subpackage GPLVault_Updater/includes
 */

/**
 * Fired during plugin deactivation.
 *
 * This class defines all code necessary to run during the plugin's deactivation.
 *
 * @since      1.0.0
 * @package    GPLVault_Updater
 * @subpackage GPLVault_Updater/includes
 * @author     GPL Vault <support@gplvault.com>
 */
class GPLVault_Updater_Deactivator {

	/**
	 * Run this method during plugin activation
	 *
	 *
	 * @since    1.0.0
	 */
	public static function deactivate() {
		foreach( array(
			'gplvault_thrice_daily_cron',
			'gplvault_six_hours_cron',
			'gplvault_hourly_cron',
			'gplvault_two_hourly_cron',
			'gplvault_five_minutes_cron',
		) as $cron_action) {
			wp_clear_scheduled_hook($cron_action);
		}

		if (! class_exists( 'GPLVault_Settings_Manager', false )) {
			require_once GV_UPDATER_INC . 'settings/class-gplvault-settings-manager.php';
		}

		if (! function_exists('GVUtil')) {
			require_once GV_UPDATER_INC . 'gplvault-functions.php';
		}

		$settings_manager = GPLVault_Settings_Manager::instance();

		try {
			if ($settings_manager->has_api_settings()) {
				GVApi()->set_initials()->deactivate();
			}
		} catch ( Exception $e ) {
		}
		GVUtil()->cleanup();

//		$settings_manager->deactivation();
	}

}
