<?php

class GPLVault_Util {
	protected static $singleton = null;
	/**
	 * @var GPLVault_Settings_Manager $settings
	 */
	protected $settings;

	public static function instance() {
		if (is_null(self::$singleton)) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	private function __construct(  ) {
		$this->settings = GPLVault_Settings_Manager::instance();
	}

	public function plugin_row_meta($links, $file) {
		if (GV_UPDATER_BASENAME === $file) {
			$row_meta = array(
				'docs'    => '<a href="' . esc_url( apply_filters( 'gplvault_docs_url', 'https://www.gplvault.com/faq' ) ) . '" aria-label="' . esc_attr__( 'View GPL Vault documentation', 'gplvault' ) . '">' . esc_html__( 'Docs', 'gplvault' ) . '</a>',
				'support' => '<a href="' . esc_url( apply_filters( 'gplvault_support_url', 'https://www.gplvault.com/contact' ) ) . '" aria-label="' . esc_attr__( 'Visit customer support', 'gplvault' ) . '">' . esc_html__( 'Support', 'gplvault' ) . '</a>',
			);

			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}

	public function actions_links( $links ) {
		$settings_link = '<a href="' . admin_url( 'admin.php?page=gplvault_settings_manager' ) . '">' . __( 'Settings', 'gplvault' ) . '</a>';

		array_unshift($links, $settings_link);

		return $links;
	}

	public function in_admin_footer() {
		if (static::is_gplvault_area()) { ?>
			<div class="gv_admin_note">
                <p><strong><?php _e('Help &amp; Support', 'gplvault') ?></strong>: <?php printf(__('If you have any question, issue with Updater Plugin or feedback, please send an email to: <a href="%1$s">help@gplvault.com</a> | <a href="%2$s" target="_blank">Terms &amp; Conditions</a> | <a href="%3$s" target="_blank">Privacy Policy</a>', 'gplvault'), 'mailto:help@gplvault.com', 'https://www.gplvault.com/terms-of-service', 'https://www.gplvault.com/privacy-and-cookie-policy') ?></p>
                <p><strong><?php _e('Do you want to grow with us?') ?></strong> <?php printf(__('Join our <strong><a href="%1$s" target="_blank">Affiliate network</a></strong>', 'gplvault'), 'https://www.gplvault.com/affiliate-area/'); ?></p>
            </div>

<?php		}
	}

	public function rename_plugins( $plugins ) {
		$gv_plugins = (array) $this->settings->get_available_plugins();

		if (! empty($gv_plugins)) {
			foreach($plugins as $key => $plugin) {
				if (array_key_exists($key, $gv_plugins) && ! empty($gv_plugins[$key]['short_name'])) {
					$plugins[$key]['Name'] = $gv_plugins[$key]['short_name'];
				}
			}
		}
		return $plugins;
	}

	public function disable_woothemes_notice() {
		if ($this->settings->disable_woo()) {
			add_filter( 'woocommerce_helper_suppress_admin_notices', '__return_true' );
		}
	}

	public static function is_gplvault_area() {
		$pages = array(
			'gplvault_plugins_manager',
			'gplvault_themes_manager',
			'gplvault_settings_manager'
		);
		$p_now = isset($_REQUEST['page']) ? sanitize_text_field($_REQUEST['page']) : '';

		return in_array($p_now, $pages, true);
	}

	public function check_status() {
       $apiObj = GVApi()->set_initials()->setGetFailurePayload();
       $status = $apiObj->status();

       if ( ! $apiObj->has_error() ) {
           GVOM()->remove_subscription_status(true);
           return;
       }

       if (! $apiObj->is_remote_error()) {
           GVOM()->remove_subscription_status(true);
           return;
       }

       GVOM()->store_subscription_status($status);
	}

	public function cleanup() {
		$main = GVMain();

		remove_action( 'init', array($this, 'disable_woothemes_notice'));

		remove_action( 'init', array($main, 'update_schema'));
		remove_action( 'init', array($main, 'gplvault_six_hours_cron'));
		remove_action( 'init', array($main, 'load_initial_schema'));

		remove_action( 'gplvault_six_hours_cron', array($main, 'update_schema'));


		remove_filter( 'rename_plugins', array($this, 'rename_plugins'));

        GPLVault_Items::instance()->remove_hooks();

        GPLVault_Settings_Manager::instance()->remove_all_schema();
	}

	public function inactive_status_notice() {
		include GV_UPDATER_STATIC_PATH . 'notices/notice-inactive.php';
	}

	public function log_schema_update( $payload, $type, $context ) {
	    /* translators: 1: schema update operation for either Theme or Plugins */
        $heading = sprintf(__('Schema update operation for: %1$s', 'gplvault'), strtoupper($context));

		/* translators: 1: status of fetched schema Error or Success */
        $status = sprintf(__('STATUS: %1$s', 'gplvault'), ucwords($type));
        $data = array();
		$message_for_saved = '';
        $saved_schema = array();

        if ('error' === trim($type)) {
            $data = $payload;
        } elseif ('theme' === trim($context)) {
            $theme_count = is_array($payload) ? count($payload) : 0;
            $data = array('payload' => 'theme-schema', 'total_themes' => $theme_count);
        } else {
	        $plugin_count = is_array($payload) ? count($payload) : 0;
	        $data = array('payload' => 'plugin-schema', 'total_plugins' => $plugin_count);
        }

        $body = __('Schema Information: ', 'gplvault') . PHP_EOL;
        $body .= gv_print_r($data, true);

        $message = $heading . PHP_EOL . $status . PHP_EOL . $body;

        gv_schema_debug($message);


		if ('success' === trim($type)) {
			$saved_schema = 'theme' === trim($context) ?
				GVOM()->get_available_themes() : GVOM()->get_available_plugins();

			$item_count = is_array($saved_schema) ? count($saved_schema) : 0;

			$message_for_saved .= sprintf(__('%1$s Schema found in the database.', 'gplvault'), strtoupper($context)) . PHP_EOL;
			$message_for_saved .= sprintf(__('Saved %2$s: %1$d', 'gplvault'), $item_count, ucwords($context) );

			gv_schema_debug($message_for_saved);
        }

	}
}
