<?php

if (! function_exists('GVEmitter')) {
	/**
	 * @return GPLVault_Updater_Loader
	 */
	function GVEmitter() {
		if (! class_exists( 'GPLVault_Updater_Loader', false) ) {
			require_once GV_UPDATER_INC . 'class-gplvault-loader.php';
		}

		return GPLVault_Updater_Loader::instance();
	}
}

/**
 * @return GPLVault_Settings_Manager
 */
function GVOM() {
	if (! class_exists( 'GPLVault_Settings_Manager', false) ) {
		require_once GV_UPDATER_INC . 'settings/class-gplvault-settings-manager.php';
	}

	return GPLVault_Settings_Manager::instance();
}

/**
 * @return GPLVault_API_Manager
 */
function GVApi() {
	if (! class_exists( 'GPLVault_API_Manager', false )) {
		require_once GV_UPDATER_INC . 'api/class-gplvault-api-manager.php';
	}

	return GPLVault_API_Manager::instance();
}

if (! function_exists( 'GVUtil') ) :
	function GVUtil() {
		if (! class_exists( 'GPLVault_Util', false )) {
			require_once GV_UPDATER_INC . 'api/class-gplvault-util.php';
		}

		return GPLVault_Util::instance();
	}
endif;

if (! function_exists( 'GVMain' ) ) :
	/**
	 * @return GPLVault_Updater
	 */
	function GVMain() {
		if (! class_exists('GPLVault_Updater', false)) {
			require GV_UPDATER_INC . 'class-gplvault-updater.php';
		}

		return GPLVault_Updater::instance();
	}
endif;

if (! function_exists( 'gv_doing_it_wrong') ) :
	function gv_doing_it_wrong($function, $message, $version) {
		// @codingStandardsIgnoreStart
		$message .= ' Backtrace: ' . wp_debug_backtrace_summary();

		if ( wp_doing_ajax() || gv_is_rest_request() ) {
			do_action( 'doing_it_wrong_run', $function, $message, $version );
			error_log( "{$function} was called incorrectly. {$message}. This message was added in version {$version}." );
		} else {
			_doing_it_wrong( $function, $message, $version );
		}
	}
endif;

if (! function_exists( 'gv_is_rest_request') ) :
	function gv_is_rest_request() {
		if ( empty( $_SERVER['REQUEST_URI'] ) ) {
			return false;
		}

		$rest_prefix         = trailingslashit( rest_get_url_prefix() );
		$is_rest_api_request = ( false !== strpos( $_SERVER['REQUEST_URI'], $rest_prefix ) ); // phpcs:disable WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

		return apply_filters( 'gplvault_is_rest_api_request', $is_rest_api_request );
	}
endif;

if (! function_exists('gv_create_files')) :
	function gv_create_files() {
		$files = array(
			array(
				'base'    => GV_UPDATER_LOG_DIR,
				'file'    => '.htaccess',
				'content' => 'deny from all',
			),
			array(
				'base'    => GV_UPDATER_LOG_DIR,
				'file'    => 'index.html',
				'content' => '',
			),
		);

		foreach ( $files as $file ) {
			if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
				$file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ); // phpcs:ignore WordPress.PHP.NoSilencedErrors.Discouraged, WordPress.WP.AlternativeFunctions.file_system_read_fopen
				if ( $file_handle ) {
					fwrite( $file_handle, $file['content'] ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_read_fwrite
					fclose( $file_handle ); // phpcs:ignore WordPress.WP.AlternativeFunctions.file_system_read_fclose
				}
			}
		}
	}
endif;

if (! function_exists( 'gv_logger')) :
	function gv_logger() {
		static $logger = null;

		$class = apply_filters( 'gplvault_logging_class', 'GPLVault_Logger' );

		if ( null !== $logger && is_string( $class ) && is_a( $logger, $class ) ) {
			return $logger;
		}

		$implements = class_implements( $class );

		if ( is_array( $implements ) && in_array( 'GPLVault_Logger_Interface', $implements, true ) ) {
			$logger = is_object( $class ) ? $class : new $class();
		} else {
			gv_doing_it_wrong(
				__FUNCTION__,
				sprintf(
				/* translators: 1: class name 2: gplvault_logging_class 3: GPLVault_Logger_Interface */
					__( 'The class %1$s provided by %2$s filter must implement %3$s.', 'gplvault' ),
					'<code>' . esc_html( is_object( $class ) ? get_class( $class ) : $class ) . '</code>',
					'<code>gplvault_logging_class</code>',
					'<code>GPLVault_Logger_Interface</code>'
				),
				'2.1.0'
			);

			$logger = is_a( $logger, 'GPLVault_Logger' ) ? $logger : new GPLVault_Logger();
		}

		return $logger;
	}
endif;

if ( ! function_exists( 'gv_log') ) :
	function gv_log($type, $message, $context = array()) {
		$logger = gv_logger();
		$type = strtolower($type);

		if (GVOM()->is_logging_disabled()) {
			return;
		}

		if(method_exists($logger, $type)) {
			$context['source'] = isset($context['source']) ? $context['source'] : 'gv-debug';
			$logger->{$type}($message, $context);
		}
	}
endif;

if (! function_exists('gv_debug')) :
	function gv_debug($message) {
		$logger = gv_logger();

		if (GVOM()->is_logging_disabled()) {
			return;
		}

		$logger->debug($message, array('source' => 'debug-log'));
	}
endif;

if (! function_exists('gv_api_debug')) :
	function gv_api_debug($message) {
		$logger = gv_logger();

		if (GVOM()->is_logging_disabled()) {
			return;
		}

		$logger->debug($message . PHP_EOL, array('source' => 'api-debug-log'));
	}
endif;

if (! function_exists('gv_schema_debug')) :
	function gv_schema_debug($message) {
		$logger = gv_logger();

		if (GVOM()->is_logging_disabled()) {
			return;
		}

		$logger->debug($message . PHP_EOL, array('source' => 'schema-debug-log'));
	}
endif;

if (! function_exists('gv_settings_debug')) :
	function gv_settings_debug($message) {
		$logger = gv_logger();

		if (GVOM()->is_logging_disabled()) {
			return;
		}

		$logger->debug($message, array('source' => 'settings-debug-log'));
	}
endif;

if (! function_exists('gv_installed_plugins_data')) {
    function gv_installed_plugins_data() {
        $gv_plugins = GVOM()->get_available_plugins();
        $result = array();

        if (empty($gv_plugins)) {
            return $result;
        }

        $installed = get_plugins();

        $active_plugins = get_option('active_plugins', array());

        if (is_multisite()) {
            $active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
        }


        foreach ($installed as $key => $plugin) {
            // consider plugins excluding GPLVault Updater and only those are supplied by gplvault.com
            if (array_key_exists($key, $gv_plugins) && strpos($key, 'gplvault-updater') === false) {
                $plugin_data = gv_take($gv_plugins[$key], array('product_id', 'version', 'type'));
                $plugin_data['installed_version'] = $plugin['Version'];
                $plugin_data['is_active'] = (in_array($key, $active_plugins) || array_key_exists($key, $active_plugins));
                $result[$key] = $plugin_data;
            }
        }

        return $result;
    }
}

if (! function_exists('gv_installed_themes_data')) {
    function gv_installed_themes_data() {
        $gv_themes = GVOM()->get_available_themes();
        $result = array();

        if (empty($gv_themes)) {
            return $result;
        }

        $installed = wp_get_themes();

        $active_theme = wp_get_theme();
        $main_theme = $active_theme->parent() ? $active_theme->parent() : $active_theme;

        foreach ($installed as $key => $theme) {
            if (array_key_exists($key, $gv_themes)) {
                $theme_data = gv_take($gv_themes[$key], array('product_id', 'version', 'type'));
                $theme_data['installed_version'] = $main_theme->get('Version');
                $theme_data['is_active'] = $main_theme->get_template() === $key;
                $result[$key] = $theme_data;
            }
        }

        return $result;
    }
}

if (! function_exists('gv_installed_themes')) {
    function gv_installed_themes() {
        return array_keys(wp_get_themes());
    }
}

if (! function_exists('gv_installed_plugins')) {
    function gv_installed_plugins() {
        return array_keys(get_plugins());
    }
}

if ( ! function_exists( 'gv_license_maybe_active' ) ) {
	function gv_license_maybe_active() {
	    return GVOM()->api_key_exists() && GVOM()->product_id_exists() && strtolower( GVOM()->get_activation_status() ) === 'activated';
	}
}
