<?php

class GPLVault_Admin {
	const SLUG_SETTINGS = 'gplvault_settings_manager';
	const SLUG_PLUGINS = 'gplvault_plugins_manager';
	const SLUG_THEME = 'gplvault_themes_manager';
	const TAB_ACTIVATION = 'gplvault_settings_activation';
	const TAB_DEACTIVATION = 'gplvault_settings_deactivation';
	const TAB_OTHERS = 'gplvault_settings_others';

	protected static $singleton = null;

	/**
	 * @var GPLVault_Settings_Manager
	 */
	protected $settings;

	/**
	 * @var GPLVault_Updater_Loader
	 */
	protected $emitter;

	protected $tab_settings = array();
	protected $instance_id;

	/**
	 * @var GPLVault_API_Manager $api
	 */
	protected $api;

	public static function instance() {
		if (is_null(self::$singleton)) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	private function __clone(  ) {}

	private function __wakeup(  ) {}

	public function __construct() {
		$this->load_dep();
		$this->set_tab_settings();
	}

	private function load_dep() {
		if (! class_exists('GPLVault_API_Manager')) {
			require_once GV_UPDATER_INC . 'api/class-gplvault-api-manager.php';
		}

		$this->emitter = GVEmitter();
		$this->settings = GVOM();
		$this->api = GPLVault_API_Manager::instance();
	}

	public function admin_scripts($hook) {
//		$page_prefix = 'gpl-vault_page_';

		wp_register_style('gv-admin-css', GV_UPDATER_STATIC_URL . 'styles/gv-admin.css', array(), GV_UPDATER_VERSION);
		wp_register_script('gv-settings-js', GV_UPDATER_STATIC_URL . 'scripts/gv-settings.js', array('jquery'), GV_UPDATER_VERSION);

		if ($hook === "toplevel_page_gplvault_settings_manager") {
			wp_enqueue_style('gv-admin-css');
			wp_enqueue_script('gv-settings-js');
		}
	}

	public function init_menu() {
		add_menu_page(
			__('GPL Vault Plugin Manager', 'gplvault'),
			__('GPL Vault', 'gplvault'),
			'manage_options',
			static::SLUG_SETTINGS,
			array($this, 'page_settings'),
			'data:image/svg+xml;base64,' . base64_encode('<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 84 88">
  <path fill="currentColor" class="cls-1" d="M5 0v19H0v40h5v29h16V76h46v12h17V0zm0 54V24h4v30zm9 5V19h-4V5h69v66H10V59zm2 24h-6v-7h6zm63 0h-7v-7h7z"/>
  <path fill="currentColor" class="cls-1" d="M73 11H18v5h50v44H18v5h55V11z"/>
  <path fill="currentColor" class="cls-1" d="M39 27.41l-2.24-4.47a17 17 0 00-9.39 16.12l5-.26A12 12 0 0139 27.41z"/>
  <path fill="currentColor" class="cls-1" d="M27.77 49.46l2.73-1.39a17 17 0 0027.5.32l1.88 1L62.29 45l-1.87-1A17 17 0 0047 21.34V19h-5v15.16a5.78 5.78 0 00-2.28 3.63L25.53 45zM35 45.77l7-3.53a4.63 4.63 0 004.81.07L53.58 46A12 12 0 0135 45.77zm21-4.18l-6.87-3.69A4.31 4.31 0 0047 34.18V26.4c5.77 1.33 11.18 7.71 9 15.19z"/>
</svg>'),
			58
		);
	}

	public function register_settings() {
		register_setting(
			GPLVault_Settings_Manager::API_SETTINGS_MAIN,
			GPLVault_Settings_Manager::API_SETTINGS_MAIN,
			null
		);

		// GPL Vault Key Activation Settings
		add_settings_section(
			GPLVault_Settings_Manager::API_SETTINGS_SECTION,
			__( 'GPL Vault License Key Activation', 'gplvault' ),
			array( $this, 'api_key_text' ),
			GPLVault_Admin::TAB_ACTIVATION
		);

		add_settings_field(
			GPLVault_Settings_Manager::API_KEY,
			__( 'API Master Key', 'gplvault' ),
			array( $this, 'api_key_field' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		add_settings_field(
			GPLVault_Settings_Manager::PRODUCT_KEY,
			__( 'Product ID', 'gplvault' ),
			array( $this, 'product_id_field' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		add_settings_field(
			'status',
			__( 'License Status', 'gplvault' ),
			array( $this, 'api_key_status' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		add_settings_field(
			'api_deactivation',
			__( 'Deactivate License', 'gplvault' ),
			array( $this, 'force_deactivate_api' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		add_settings_field(
			'check_api_status',
			__( 'Check License Status', 'gplvault' ),
			array( $this, 'check_api_status' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		add_settings_field(
			'cleanup_api_settings',
			__( 'Cleanup Local Settings', 'gplvault' ),
			array( $this, 'cleanup_api_settings' ),
			GPLVault_Admin::TAB_ACTIVATION,
			GPLVault_Settings_Manager::API_SETTINGS_SECTION
		);

		// Other GPL Vault Settings
		register_setting(
			GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY,
			GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY,
			array( $this, 'gplvault_settings_extra' )
		);

		add_settings_section(
			GPLVault_Settings_Manager::EXTRA_SECTION,
			__( 'Extra Settings', 'gplvault' ),
			array( $this, 'extra_settings_section' ),
			GPLVault_Admin::TAB_OTHERS
		);

		add_settings_field(
			GPLVault_Settings_Manager::EXTRA_WOO_NOTICE,
			__( 'Disable Wootheme Notice', 'gplvault' ),
			array( $this, 'disable_woo_field' ),
			GPLVault_Admin::TAB_OTHERS,
			GPLVault_Settings_Manager::EXTRA_SECTION
		);

		add_settings_field(
			GPLVault_Settings_Manager::EXTRA_LOG,
			__( 'Disable Logging', 'gplvault' ),
			array( $this, 'disable_log_field' ),
			GPLVault_Admin::TAB_OTHERS,
			GPLVault_Settings_Manager::EXTRA_SECTION
		);
	}

	//
	//  Callbacks for Settings API starts
	//

	public function cleanup_api_settings() {
		$api_settings = $this->settings->get_api_settings();

		$disabled = ! empty($api_settings) ? '' : ' disabled';

		$tip = sprintf(__('If you are having troubles with cleaning up activations or activations on both main server and your site are not in sync, please delete entries on main server from %1$s against this domain, then clean up local settings and try to activate your license once again. We are sorry for your inconvenience. This option should be used only if your activation is messed up.', 'gplvault'),
			'<a href="https://www.gplvault.com/my-account/api-keys/" target="_blank">www.gplvault.com</a>');

		echo '<button id="gv_cleanup_settings" type="button" class="button button-danger" data-action="gv_cleanup_settings" data-nonce="'. wp_create_nonce('gv_cleanup_settings') .'"' .$disabled. '>'. __('Clean up local settings', 'gplvault') .'</button>';
		echo '<div class="description">' . $tip . '</div>';
	}

	public function check_api_status() {
		$api_settings = $this->settings->get_api_settings();

		$disabled = ! empty($api_settings) ? '' : ' disabled';

		$tip = __('Check API Status from the server.', 'gplvault');
		echo '<button id="gv_check_api" type="button" class="button button-primary" data-action="gv_check_api" data-nonce="'. wp_create_nonce('gv_check_api') .'"' .$disabled. '>'. __('Check License', 'gplvault') .'</button>';
		echo '<div class="description">' . $tip . '</div>';
	}

	public function force_deactivate_api() {
		$api_settings = $this->settings->get_api_settings();

		$disabled = ! empty($api_settings) ? '' : ' disabled';

		$tip = __('Deactivate your current API activation on the main server <code>www.gplvault.com</code>', 'gplvault');
		echo '<button id="gv_deactivate_api" type="button" class="button button-danger" data-action="gv_deactivate_api" data-nonce="'. wp_create_nonce('gv_deactivate_api') .'"' .$disabled. '>'. __('Deactivate License', 'gplvault') .'</button>';
		echo '<div class="description">' . $tip . '</div>';
	}

	public function api_key_text() {
		echo '<div id="gv_reports"></div>';
	}

	public function api_key_status() {
		$api_key_status = trim($this->settings->get(GPLVault_Settings_Manager::DEACTIVATION_KEY));
		$status_check_text = empty($api_key_status) ? 'Deactivated' : $api_key_status;

		echo '<div id="gv_api_status_text">' . $status_check_text .'</div>';
	}

	public function api_key_field() {
		$api_key_option = $this->settings->get_api_key();
		$api_key = empty($api_key_option) ? '' : $api_key_option;
        $report_class = $api_key_option ? 'api-has-activation': 'api-no-activation';

		echo "<input class='gv_api_field ". $report_class ."' id='". GPLVault_Settings_Manager::API_KEY ."' name='" . GPLVault_Settings_Manager::API_SETTINGS_MAIN . "[" . GPLVault_Settings_Manager::API_KEY ."]' type='password' value='" . $api_key . "' required />";
		if ( $api_key_option ) {
			echo "<span class='gv_mark_icon dashicons dashicons-yes' style='color: #66ab03;'></span>";
		} else {
			echo "<span class='gv_mark_icon dashicons dashicons-no' style='color: #ca336c;'></span>";
		}
		echo '<div class="description">'. __('Please, enter the Master Key from My Account page of GPL Vault', 'gplvault') .'</div>';
	}

	public function product_id_field() {
		$product_key_option = $this->settings->get_product_id();
		$product_id = empty($product_key_option) ? '' : $product_key_option;
		$report_class = $product_key_option ? 'api-has-activation': 'api-no-activation';

		echo "<input class='gv_api_field " . $report_class ."' id='". GPLVault_Settings_Manager::PRODUCT_KEY . "' name='" . GPLVault_Settings_Manager::API_SETTINGS_MAIN . "[" . GPLVault_Settings_Manager::PRODUCT_KEY ."]' type='text' value='" . $product_id . "' required />";
		if ( $product_key_option ) {
			echo "<span class='gv_mark_icon dashicons dashicons-yes' style='color: #66ab03;'></span>";
		} else {
			echo "<span class='gv_mark_icon dashicons dashicons-no' style='color: #ca336c;'></span>";
		}
		echo '<div class="description">'. __('Please, enter the Product ID listed on My Account page of GPL Vault', 'gplvault') .'</div>';
	}

    public function ajax_activate_api_key() {
        if (! function_exists('GVOM')) {
            require_once GV_UPDATER_INC . 'gplvault-functions.php';
        }

        // check request validity
        $verified = check_ajax_referer('gv_cleanup_settings', 'security', false);
        if (! $verified) {
            wp_send_json_error(array(
                'error' => __('Request is not authorized and recognized as insecure or malicious request to the system.', 'gplvault'),
            ));
        }

        $api_key = ! empty($_POST['api_key']) ? gv_clean($_POST['api_key']) : null;
        $product_id = ! empty($_POST['product_id']) ? gv_clean($_POST['product_id']) : null;

        if (empty($api_key) && empty($product_id)) {
            wp_send_json_error(array(
                'error' => __( 'Both Product ID and API Key field is required', 'gplvault' ),
            ));
        }

        $apiObj = GVApi()->set_api_key($api_key)->set_product_id($product_id)->setGetFailurePayload();
        // check api status
        @gv_settings_debug(esc_html__('Checking License status...', 'gplvault'));
        $status = $apiObj->status();

        if (isset($status['code'])) {
            $message = __("API Activation did not take place, because unknown error occurred.", 'gplvault');
            if (isset($status['error'])) {
                $message = $status['error'];
            } elseif (isset($status['message'])) {
                $message = $status['message'];
            }

            unset($apiObj);
            wp_send_json_error(array( 'message' => $message));
        }

        // TODO: heavily dependant on WooCommerce API Manager
        if ( isset($status['status_check']) && $status['status_check'] === 'inactive' ) {
            @gv_settings_debug(esc_html__('License status checked successfully!', 'gplvault') . PHP_EOL);

            // API status is inactive and going to activate the Key
            @gv_settings_debug(esc_html__('Activating License on the server...', 'gplvault'));
            $activation_response = $apiObj->activate();

            // activation failed
            if (isset($activation_response['code'])) {
                $message = __('Unknown error occurred during activation process', 'gplvault');
                if (isset($activation_response['error'])) {
                    $message = $activation_response['error'];
                } elseif (isset($activation_response['message'])) {
                    $message = $activation_response['message'];
                }

                @gv_settings_debug(esc_html__('License activation failed: ', 'gplvault') . PHP_EOL);
                @gv_settings_debug(gv_print_r($activation_response) . PHP_EOL);

                unset($apiObj);
                wp_send_json_error(array( 'message' => $message));
            }
            GVOM()->save_api_settings(array(
                'api_key' => $api_key,
                'product_id' => $product_id,
            ));

            GVOM()->enable_activation_status();

            do_action( 'gv_api_license_activated');

            @gv_settings_debug(__('License activated successfully! Please, check your Dashboard under "API Keys" section on gplvault.com.', 'gplvault') . PHP_EOL);
            unset($apiObj);
            wp_send_json_success(array( 'message' => $activation_response['message']));
        }

        GVOM()->save_api_settings(array(
            'api_key' => $api_key,
            'product_id' => $product_id,
        ));

        GVOM()->enable_activation_status();

        @gv_settings_debug(esc_html__('License key already activated for this site.', 'gplvault') . PHP_EOL);
        unset($apiObj);
        wp_send_json_success(array('message' => __('Your license is already activated!', 'gplvault')));
    }

	public function gplvault_settings_extra() {
		$settings_extra = (array) $this->settings->extra_settings(array());
		$woo_param = 0;
		$disable_log = 'no';
		if ($_REQUEST['option_page'] === GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY) {
			@gv_settings_debug(esc_html__('Processing extra settings...', 'gplvault'));

			if (isset($_POST[GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY])) {
				$form_data = $_POST[GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY];
				$woo_param = isset($form_data[GPLVault_Settings_Manager::EXTRA_WOO_NOTICE]) ? intval($form_data[GPLVault_Settings_Manager::EXTRA_WOO_NOTICE]) : 0;
				$disable_log = isset($form_data[GPLVault_Settings_Manager::EXTRA_LOG]) ? gv_clean($form_data[GPLVault_Settings_Manager::EXTRA_LOG]) : 'no';
			}


			$settings_extra[GPLVault_Settings_Manager::EXTRA_WOO_NOTICE] = $woo_param;
			$settings_extra[GPLVault_Settings_Manager::EXTRA_LOG] = $disable_log;

			@gv_settings_debug(esc_html__('Saving extra settings:', 'gplvault') . PHP_EOL);
            @gv_settings_debug(gv_print_r($settings_extra, true) . PHP_EOL);

			return $settings_extra;
		}
	}

	public function extra_settings_section() {

	}

	public function disable_woo_field() {
		echo '<label for="' . esc_attr( GPLVault_Settings_Manager::EXTRA_WOO_NOTICE ) . '"><input type="checkbox" id="' . esc_attr( GPLVault_Settings_Manager::EXTRA_WOO_NOTICE ) . '" name="' . esc_attr(GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY) . '['. esc_attr( GPLVault_Settings_Manager::EXTRA_WOO_NOTICE ) . ']" value="1"';
		echo checked( intval($this->settings->disable_woo()), 1 );
		echo '/>';
		?><span class="description"><?php esc_html_e( 'Disable WooCommerce Updater notice', 'gplvault' ); ?></span></label>
		<?php
	}

	public function disable_log_field() {
		echo '<label for="' . esc_attr( GPLVault_Settings_Manager::EXTRA_LOG ) . '"><input type="checkbox" id="' . esc_attr( GPLVault_Settings_Manager::EXTRA_LOG ) . '" name="' . esc_attr(GPLVault_Settings_Manager::EXTRA_SETTINGS_KEY) . '['. esc_attr( GPLVault_Settings_Manager::EXTRA_LOG ) . ']" value="yes"';
		echo checked( gv_clean($this->settings->disable_log()), 'yes' );
		echo '/>';
		?><span class="description"><?php esc_html_e( 'Disable log', 'gplvault' ); ?></span></label>
		<?php
	}

	//
	//  Callbacks for Settings API ends
	//

	public function page_settings() {
		include( GV_UPDATER_STATIC_PATH . 'partials/settings.php');
	}

	public function set_tab_settings() {
		$this->tab_settings = array(
			static::TAB_ACTIVATION => __('License Activation', 'gplvault'),
			static::TAB_OTHERS => __('Other Options', 'gplvault'),
		);

		return $this;
	}

	public function get_tab_settings() {
		if (empty($this->tab_settings)) {
			$this->set_tab_settings();
		}
		return $this->tab_settings;
	}

	/**
	 * Ajax hook to deactivate API License
	 */
	public function ajax_deactivate_license() {
		if (! function_exists('GVApi')) {
			require_once GV_UPDATER_INC . 'gplvault-functions.php';
		}

		$verified = check_ajax_referer('gv_deactivate_api', false, false);

		if (! $verified) {
			wp_send_json_error(array(
				'error' => __('Ajax request was initiated from unauthorised source.', 'gplvault'),
			));
		}

		$response = GVApi()->set_initials()->setGetFailurePayload()->deactivate();

        if (isset($response['code'])) {
            $message = __('Unknown error occurred during deactivation process', 'gplvault');
            if (isset($response['error'])) {
                $message = $response['error'];
            } elseif (isset($response['message'])) {
                $message = $response['message'];
            }

            wp_send_json_error(array( 'error' => $message));
        }

        if (isset($response['success']) && $response['success']) {
            $payload = array(
                'status' => $response['deactivated'] ? 'deactivated' : 'not deactivated',
                'activations' => $response['data']['total_activations_purchased'],
                'used' => $response['data']['total_activations'],
                'remaining' => $response['data']['activations_remaining'],
            );

            GVOM()->disable_activation_status();
            GVOM()->remove_api_key();
            GVOM()->remove_all_schema();

            do_action('gv_api_license_deactivated');

            wp_send_json_success($payload);
        } else {
            $error_response = array(
                'error' => __('API Call could not be made for unknown reason', 'gplvault'),
            );
            wp_send_json_error($error_response);
        }
	}

	/*
	 * Ajax handler to check API status
	 */
	public function ajax_gv_check_api() {
		if (! function_exists('GVApi')) {
			require_once GV_UPDATER_INC . 'gplvault-functions.php';
		}

		$verified = check_ajax_referer('gv_check_api', false, false);

		if (! $verified) {
			wp_send_json_error(array(
				'error' => __('Request is not authorized and recognized as insecure or malicious request to the system.', 'gplvault'),
			));
		}

		$apiManager = GVApi()->set_initials()->setGetFailurePayload();

		$response = $apiManager->status();

		if ($apiManager->has_error()) {
		    if ($apiManager->is_system_error()) {
		        GVOM()->remove_subscription_status(true);
            } else {
		        GVOM()->subscription_status($response);
            }

            $message = __('Unknown error occurred during deactivation process', 'gplvault');
            if (isset($response['error'])) {
                $message = $response['error'];
            } elseif (isset($response['message'])) {
                $message = $response['message'];
            }

            wp_send_json_error(array( 'error' => $message));
        }

        if (isset($response['success']) && $response['success']) {
            $payload = array(
                'status' => $response['status_check'],
                'activations' => $response['data']['total_activations_purchased'],
                'used' => $response['data']['total_activations'],
                'remaining' => $response['data']['activations_remaining'],
                'activated' => $response['data']['activated'],
            );

            GVOM()->remove_subscription_status(true);

            if ($response['data']['activated']) {
                if (! GVOM()->license_is_activated()) {
                    GVOM()->enable_activation_status();
                }
            } else {
                GVOM()->disable_activation_status();
            }

            wp_send_json_success($payload);
        } else {
            $error_response = array(
                'error' => __('API Call could not be made for unknown reason', 'gplvault'),
            );
            wp_send_json_error($error_response);
        }
	}


	// Ajax handler for local settings clean up
	public function ajax_clear_local_settings() {
		if (! function_exists('GVOM')) {
			require_once GV_UPDATER_INC . 'gplvault-functions.php';
		}

		$verified = check_ajax_referer('gv_cleanup_settings', false, false);

		if (! $verified) {
			wp_send_json_error(array(
				'error' => __('Request is not authorized and recognized as insecure or malicious request to the system.', 'gplvault'),
			));
		}

		GVOM()->remove_all_schema();
		GVOM()->disable_activation_status();
		GVOM()->remove_api_key();

		wp_send_json_success(array(
			'message' => __('Local settings is cleared!', 'gplvault'),
        ));
    }

	public function maybe_deferred_package( $options ) {
		$package = $options['package'];
		if ( false !== strrpos( $package, 'gv_delayed_download' ) && false !== strrpos( $package, 'gv_item_id' ) ) {
			parse_str( parse_url( $package, PHP_URL_QUERY ), $vars );
			if ( $vars['gv_item_id'] ) {
				$options['package'] = $this->api->set_initials()->download( array('product_id' => $vars['gv_item_id']) );
			}
		}

		return $options;
    }
}
