jQuery(document).ready(function($) {
    var $doc, $gv_check_api, $report_box, $gv_deactivate_api,
        $gv_cleanup_settings, $gplvault_settings_main, $save_api_settings;

    $doc = $(document);
    $gv_check_api = $doc.find('#gv_check_api');
    $report_box = $doc.find('#custom_errors_container');
    $save_api_settings = $doc.find('#save_api_settings');
    $gv_deactivate_api = $doc.find('#gv_deactivate_api');
    $gv_cleanup_settings = $doc.find('#gv_cleanup_settings');
    $gplvault_settings_main = $doc.find('#gplvault_settings_main');

    $report_box.on('click', 'button.notice-dismiss', function(e) {
       e.preventDefault();
       var $instance = $(this);

       $instance.closest('.is-dismissible').remove();
    });

    function showReport(payload, type) {
        type = type || 'success';
        var html = '', notice;

        notice = '<div class="notice notice-' + type + ' is-dismissible"><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button>';
        if (type === 'success') {
            if ($.isArray(payload)) {
                payload.forEach(function(item) {
                    html += '<p>';
                    html += '<strong>' + item.label + ':</strong>' + "&nbsp;&nbsp;";
                    html += '<span>' + item.text + '</span>';
                    html += '</p>';
                });
            } else {
                html += '<p>' + payload + '</p>';
            }

            // notice = '<div class="notice notice-success is-dismissible"><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button>';
            notice += html;
            notice += '</div>';

            return notice;
        }

        notice += '<p>' + payload + '</p>';
        notice += '</div>';

        return notice;

    }


    $save_api_settings.on('click', function(event) {
        event.preventDefault();
        var $form, $instance,
            $xhr, valid,
            $requiredInputs;

        $instance = $(this);
        $form = $instance.closest('form');
        valid = false;

        var api_key = $form.find('#api_key').val().trim();
        var product_id = $form.find('#product_id').val().trim();
        var nonce = $instance.data('nonce');

        $requiredInputs = $form.find('.gv_api_field');

        $requiredInputs.each(function($_, elem) {
            var elemValue = elem.value.trim();

            if (elemValue.length > 0) {
                valid = true;
                $(elem).removeClass('gv-error');
            } else {
               valid = false;
               $(elem).removeClass('gv-error').addClass('gv-error');
            }
        });

        if ( api_key.length === 0 || product_id.length === 0) {
            $report_box.html(showReport('Both API Key and Product ID fields are required.', 'error'));
            return false;
        }

        $instance.addClass('updating-message');
        $xhr = $.ajax(ajaxurl, {
            method: 'POST',
            data: {
                security: nonce,
                action: 'gv_activate_api',
                product_id: product_id,
                api_key: api_key,
            }
        });

        $xhr.done(function(response) {
            if (response.success) {
                $doc.trigger('gv.api_activated');
                $report_box.html(showReport(response.data.message));
            } else {
                $report_box.html(showReport(response.data.message, 'error'));
            }
        });

        $xhr.always(function() {
            $instance.removeClass('updating-message');
        });
    });

    // check api status
    $gv_check_api.on('click', function(evt) {
        evt.preventDefault();
        var nonce, action, xhr, $instance;
        $instance = $(this);
        nonce = $instance.data('nonce');
        action = $instance.data('action');

        $instance.addClass('updating-message');

        xhr = $.ajax(ajaxurl, {
                method: 'POST',
                data: {
                    action: action,
                    _ajax_nonce: nonce,
                }
            });

        xhr.done(function(response) {
           if (response.success) {
               var resp_data = response.data;
               var reportPayload = [
                   {label: 'Status', text: resp_data.status},
                   {label: 'Purchased Activations', text: resp_data.activations},
                   {label: 'Total Activated', text: resp_data.used},
                   {label: 'Remaining Activations', text: resp_data.remaining},
                   {label: 'Activated?', text: resp_data.activated ? 'Activated' : 'Deactivated'},
               ];

               $report_box.html(showReport(reportPayload));
           } else {
               $report_box.html(showReport(response.data.error, 'error'));
           }
        });

        xhr.always(function() {
            $instance.removeClass('updating-message');
        })
    });

    // deactivate license
    $gv_deactivate_api.on('click', function(evt) {
        evt.preventDefault();
        var nonce, action, xhr, $instance;
        $instance = $(this);
        nonce = $instance.data('nonce');
        action = $instance.data('action');

        $instance.addClass('updating-message');

        xhr = $.ajax(ajaxurl, {
            method: 'POST',
            data: {
                action: action,
                _ajax_nonce: nonce,
            }
        });

        xhr.done(function(response) {
            if (response.success) {
                var resp_data = response.data;
                var reportPayload = [
                    {label: 'Status', text: resp_data.status},
                    {label: 'Purchased Activations', text: resp_data.activations},
                    {label: 'Total Activated', text: resp_data.used},
                    {label: 'Remaining Activations', text: resp_data.remaining},
                    {label: 'Activated?', text: resp_data.activated ? 'Activated' : 'Deactivated'},
                ];

                $('#gv_api_status_text').html('Deactivated');
                $doc.trigger('gv.api_deactivated');

                $report_box.html(showReport(reportPayload));

            } else {
                $report_box.html(showReport(response.data.error, 'error'));
            }
        });

        xhr.always(function() {
            $instance.removeClass('updating-message');
        })
    });


    $doc.on('gv.api_deactivated', function(event) {
        $('#gv_api_status_text').html('Deactivated');
        $gplvault_settings_main.find('.gv_api_field').val('');
        $gplvault_settings_main.find('.gv_mark_icon').removeClass('dashicons-yes').addClass('dashicons-no').css('color', '#ca336c');

        // disable button
        $gv_deactivate_api.prop('disabled', true);
        $gv_cleanup_settings.prop('disabled', true);
        $gv_check_api.prop('disabled', true);

        // enable submit button
        $save_api_settings.prop('disabled', false);
    });

    $doc.on('gv.api_activated', function(event) {

        $('#gv_api_status_text').html('Activated');
        $gplvault_settings_main.find('.gv_mark_icon').removeClass('dashicons-no').addClass('dashicons-yes').css('color', '#66ab03');

        // enable buttons
        $gv_deactivate_api.prop('disabled', false);
        $gv_cleanup_settings.prop('disabled', false);
        $gv_check_api.prop('disabled', false);

        // disable submit button
        $save_api_settings.prop('disabled', true);
    });

    // clean up local settings
    $gv_cleanup_settings.on('click', function(evt) {
        evt.preventDefault();
        var nonce, action, xhr, $instance;
        $instance = $(this);
        nonce = $instance.data('nonce');
        action = $instance.data('action');

        $instance.addClass('updating-message');

        xhr = $.ajax(ajaxurl, {
            method: 'POST',
            data: {
                action: action,
                _ajax_nonce: nonce,
            }
        });

        xhr.done(function(response) {
            if (response.success) {
                var resp_data = response.data;

                $doc.trigger('gv.api_deactivated');
                // $('#gv_api_status_text').html('Deactivated');
                // $gplvault_settings_main.find('.gv_api_field').val('');
                // $gplvault_settings_main.find('.gv_mark_icon').removeClass('dashicons-yes').addClass('dashicons-no').css('color', '#ca336c');

                $report_box.html(showReport(resp_data.message));

            } else {
                $report_box.html(showReport(response.data.error, 'error'));
            }
        });

        xhr.always(function() {
            $instance.removeClass('updating-message');
        })
    });
});